@@ -16,12 +16,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gnome-network-displays-config.h"
-#include "nd-wfd-p2p-sink.h"
-#include "wfd/wfd-server.h"
 #include "wfd/wfd-client.h"
 #include "wfd/wfd-media-factory.h"
+#include "wfd/wfd-server.h"
+#include "gnome-network-displays-config.h"
 #include "nd-firewalld.h"
+#include "nd-wfd-p2p-sink.h"
 
 struct _NdWFDP2PSink
 {
@@ -36,8 +36,8 @@ struct _NdWFDP2PSink
   NMWifiP2PPeer      *nm_peer;
   NMActiveConnection *nm_ac;
 
-  GStrv               missing_video_codec;
-  GStrv               missing_audio_codec;
+  GtkStringList      *missing_video_codec;
+  GtkStringList      *missing_audio_codec;
   char               *missing_firewall_zone;
 
   WfdServer          *server;
@@ -131,15 +131,19 @@ nd_wfd_p2p_sink_get_property (GObject    *object,
     case PROP_MATCHES:
       {
         g_autoptr(GPtrArray) res = NULL;
-        const char *name;
+        const char *hw_addr;
         res = g_ptr_array_new_with_free_func (g_free);
 
         /* Should not usually happen, but it can if something is holding on
          * to the sink. So guard against NULL being returned if the peer
          * object is not valid anymore. */
-        name = nm_wifi_p2p_peer_get_name (sink->nm_peer);
-        if (name)
-          g_ptr_array_add (res, g_strdup (name));
+        hw_addr = nm_wifi_p2p_peer_get_hw_address (sink->nm_peer);
+        if (hw_addr)
+          {
+            gchar *addr = g_utf8_strup (hw_addr, -1);
+            g_debug ("NdWFDP2PSink: Adding P2P MAC %s to match list", addr);
+            g_ptr_array_add (res, addr);
+          }
 
         g_value_take_boxed (value, g_steal_pointer (&res));
         break;
@@ -154,11 +158,11 @@ nd_wfd_p2p_sink_get_property (GObject    *object,
       break;
 
     case PROP_MISSING_VIDEO_CODEC:
-      g_value_set_boxed (value, sink->missing_video_codec);
+      g_value_set_object (value, sink->missing_video_codec);
       break;
 
     case PROP_MISSING_AUDIO_CODEC:
-      g_value_set_boxed (value, sink->missing_audio_codec);
+      g_value_set_object (value, sink->missing_audio_codec);
       break;
 
     case PROP_MISSING_FIREWALL_ZONE:
@@ -228,8 +232,8 @@ nd_wfd_p2p_sink_finalize (GObject *object)
   g_clear_object (&sink->nm_device);
   g_clear_object (&sink->nm_peer);
 
-  g_clear_pointer (&sink->missing_video_codec, g_strfreev);
-  g_clear_pointer (&sink->missing_audio_codec, g_strfreev);
+  g_clear_object (&sink->missing_video_codec);
+  g_clear_object (&sink->missing_audio_codec);
   g_clear_pointer (&sink->missing_firewall_zone, g_free);
 
   G_OBJECT_CLASS (nd_wfd_p2p_sink_parent_class)->finalize (object);
@@ -522,18 +526,18 @@ nd_wfd_p2p_sink_sink_start_stream (NdSink *sink)
 
   have_basic_codecs = wfd_get_missing_codecs (&missing_video, &missing_audio);
 
-  g_clear_pointer (&self->missing_video_codec, g_strfreev);
-  g_clear_pointer (&self->missing_audio_codec, g_strfreev);
+  g_clear_object (&self->missing_video_codec);
+  g_clear_object (&self->missing_audio_codec);
 
-  self->missing_video_codec = g_strdupv (missing_video);
-  self->missing_audio_codec = g_strdupv (missing_audio);
+  self->missing_video_codec = gtk_string_list_new ((const char *const *) missing_video);
+  self->missing_audio_codec = gtk_string_list_new ((const char *const *) missing_audio);
 
   g_object_notify (G_OBJECT (self), "missing-video-codec");
   g_object_notify (G_OBJECT (self), "missing-audio-codec");
 
   if (!have_basic_codecs)
     {
-      g_warning ("Essential codecs are missing!");
+      g_warning ("NdWFDP2PSinkEssential codecs are missing!");
       self->state = ND_SINK_STATE_ERROR;
       g_object_notify (G_OBJECT (self), "state");
 
@@ -643,6 +647,20 @@ nd_wfd_p2p_sink_get_peer (NdWFDP2PSink * sink)
   return sink->nm_peer;
 }
 
+/**
+ * nd_wfd_p2p_sink_get_state
+ * @sink: a #NdWFDP2PSink
+ *
+ * Retrieve the #NdSinkState.
+ *
+ * Returns: (transfer none): The #NdSinkState
+ */
+NdSinkState
+nd_wfd_p2p_sink_get_state (NdWFDP2PSink *sink)
+{
+  return sink->state;
+}
+
 NdWFDP2PSink *
 nd_wfd_p2p_sink_new (NMClient *client, NMDevice *device, NMWifiP2PPeer * peer)
 {
